! ############################################################
!            Satellite Data Simulation Unit v2
!                     - mie_lut_rdr -
!
! This subroutine find the extinction/scattering coefficients for
! given T, hydrometeor water content, and hydrometeor number 
! concentration using the Mie LUTs. 
!
!                  Updated from mie-lut.f by H. M. (May, 2009)
!                Updated for SDSU v.2.1.0 by H. M. (Aug, 2009)
!
 Subroutine mie_lut_rdr(Whydr,x1,x2,pts_x1,pts_x2, &
                        kext_lut,salb_lut,pbck_lut, &
                        kext,salb,pbck)
   Use RDRparameters
   Implicit NONE
!
! I/O parameters
! 
   real, intent(in)  :: Whydr,x1,x2
   real, dimension(:), intent(in) :: pts_x1,pts_x2
   real, dimension(:,:,:), intent(in) :: kext_lut,salb_lut,pbck_lut
   real, intent(out) :: kext,salb,pbck
!
! Local variables
!
   real, dimension(:,:), allocatable :: hydr2_lut
   real, dimension(:), allocatable :: hydr1_lut
   integer :: nph,nx1,nx2
   real :: f1,fW,f2
   Integer :: i,j,k
!
   kext = 0. ; salb = 0. ; pbck = 0.
   nph = size(kext_lut,dim=1)
   nx1 = size(kext_lut,dim=2)
   nx2 = size(kext_lut,dim=3)
   allocate ( hydr2_lut(nph,nx1), hydr1_lut(nph) )
!
! Skip if x2 is out of range except when nx2 = 1.
!
   If (nx2 > 1 .and. x2 < pts_x2(1))   Return
   If (nx2 > 1 .and. x2 > pts_x2(nx2)) Return
!
! Skip if x1 is out of range except where nx1 = 1.
!
   If (nx1 > 1 .and. x1  < pts_x1(1))    Return
   If (nx1 > 1 .and. x1  > pts_x1(nx1))  Return
!      
! Skip if W is out of range.
!
   If (Whydr < pts_hydro(1))   Return
   If (Whydr > pts_hydro(nph)) Return
!
! /////////////// kext //////////////////
!
! Locate x2
!
   Do k = 2, nx2
      If (x2 > pts_x2(k)) cycle
      f2 = (pts_x2(k)-x2) / (pts_x2(k)-pts_x2(k-1))
      hydr2_lut = f2*kext_lut(:,:,k-1) + (1.-f2)*kext_lut(:,:,k)
      exit
   End do
   If ( nx2 == 1 )  hydr2_lut = kext_lut(:,:,1)
!
! Locate x1
!
   Do j = 2 , nx1
      If (x1 > pts_x1(j)) cycle
      f1 = (pts_x1(j)-x1) / (pts_x1(j)-pts_x1(j-1))
      hydr1_lut = f1*hydr2_lut(:,j-1) + (1.-f1)*hydr2_lut(:,j)
      exit
   End do
   If ( nx1 == 1 ) hydr1_lut = hydr2_lut(:,1)
!
! Locate W
!
   Do i = 2 , nph
      If (Whydr > pts_hydro(i)) cycle
      fW = (pts_hydro(i)-Whydr) / (pts_hydro(i)-pts_hydro(i-1))
      kext = fW*hydr1_lut(i-1) + (1.-fW)*hydr1_lut(i)
      exit
   End do
!
! /////////////// salb //////////////////
!
! Locate x2
!
   Do k = 2, nx2
      If (x2 > pts_x2(k)) cycle
      f2 = (pts_x2(k)-x2) / (pts_x2(k)-pts_x2(k-1))
      hydr2_lut = f2*salb_lut(:,:,k-1) + (1.-f2)*salb_lut(:,:,k)
      exit
   End do
   If ( nx2 == 1 )  hydr2_lut = salb_lut(:,:,1)
!
! Locate x1
!
   Do j = 2 , nx1
      If (x1 > pts_x1(j)) cycle
      f1 = (pts_x1(j)-x1) / (pts_x1(j)-pts_x1(j-1))
      hydr1_lut = f1*hydr2_lut(:,j-1) + (1.-f1)*hydr2_lut(:,j)
      exit
   End do
   If ( nx1 == 1 ) hydr1_lut = hydr2_lut(:,1)
!
! Locate W
!
   Do i = 2 , nph
      If (Whydr > pts_hydro(i)) cycle
      fW = (pts_hydro(i)-Whydr) / (pts_hydro(i)-pts_hydro(i-1))
      salb = fW*hydr1_lut(i-1) + (1.-fW)*hydr1_lut(i)
      exit
   End do
!
! /////////////// pbck //////////////////
!
! Locate x2
!
   Do k = 2, nx2
      If (x2 > pts_x2(k)) cycle
      f2 = (pts_x2(k)-x2) / (pts_x2(k)-pts_x2(k-1))
      hydr2_lut = f2*pbck_lut(:,:,k-1) + (1.-f2)*pbck_lut(:,:,k)
      exit
   End do
   If ( nx2 == 1 )  hydr2_lut = pbck_lut(:,:,1)
!
! Locate x1
!
   Do j = 2 , nx1
      If (x1 > pts_x1(j)) cycle
      f1 = (pts_x1(j)-x1) / (pts_x1(j)-pts_x1(j-1))
      hydr1_lut = f1*hydr2_lut(:,j-1) + (1.-f1)*hydr2_lut(:,j)
      exit
   End do
   If ( nx1 == 1 ) hydr1_lut = hydr2_lut(:,1)
!
! Locate W
!
   Do i = 2 , nph
      If (Whydr > pts_hydro(i)) cycle
      fW = (pts_hydro(i)-Whydr) / (pts_hydro(i)-pts_hydro(i-1))
      pbck = fW*hydr1_lut(i-1) + (1.-fW)*hydr1_lut(i)
      exit
   End do
!
   deallocate ( hydr2_lut, hydr1_lut )
   Return
 End Subroutine mie_lut_rdr
